/*
 * Copyright 2012 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.bean;

import javax.sql.DataSource;
import org.apache.commons.dbcp.BasicDataSource;
import org.ferris.yahoo.finance.jmce.handler.*;

/**
 *
 * @author mjr2958
 */
public class BeanFactory 
{
    private static PrimaryKeyHandlerByJava primaryKeyHandler;
    public static PrimaryKeyHandler getPrimaryKeyHandler() {
        if (primaryKeyHandler == null) {
            primaryKeyHandler = new PrimaryKeyHandlerByJava();
        }
        return primaryKeyHandler;
    }
    
    private static DayEndPriceHandlerBySql dayEndPriceHandler;
    public static DayEndPriceHandler getDayEndPriceHandler()
    {        
        if (dayEndPriceHandler == null) {
            dayEndPriceHandler = new DayEndPriceHandlerBySql();
            dayEndPriceHandler.setCalendarHandler(getCalendarHandler());
            dayEndPriceHandler.setDataSource(getDataSource());
            dayEndPriceHandler.setPrimaryKeyHandler(getPrimaryKeyHandler());
        }
        return dayEndPriceHandler;
    }
    
    private static CmeGroupHandlerByJson cmeGroupHandler;
    public static CmeGroupHandler getCmeGroupHandler()
    {
        if (cmeGroupHandler == null) {
            cmeGroupHandler = new CmeGroupHandlerByJson();
        }
        return cmeGroupHandler;
    }
    
    
    private static BasicDataSource dataSource;    
    public static DataSource getDataSource()
    {
        if (dataSource == null) {
            dataSource = new BasicDataSource();         
            dataSource.setDriverClassName("org.postgresql.Driver");
            dataSource.setUsername(System.getProperty("org.ferris.yahoo.finance.jmce.property.user"));
            dataSource.setPassword(System.getProperty("org.ferris.yahoo.finance.jmce.property.pass"));         
            dataSource.setUrl(System.getProperty("org.ferris.yahoo.finance.jmce.property.url"));
            dataSource.setInitialSize(1);
            dataSource.setMaxActive(1);        
        }
        return dataSource;
    }
    
    private static YearEndPriceHandlerBySql yearEndPriceHandler;
    public static YearEndPriceHandler getYearEndPriceHandler()
    {
        if (yearEndPriceHandler == null) {
            yearEndPriceHandler = new YearEndPriceHandlerBySql();
            yearEndPriceHandler.setCalendarHandler(getCalendarHandler());
            yearEndPriceHandler.setDataSource(getDataSource());
            yearEndPriceHandler.setPrimaryKeyHandler(getPrimaryKeyHandler());
        }
        return yearEndPriceHandler;
    }
    
    private static StandardAndPoors500HandlerByScrape standardAndPoorsHandler;
    public static StandardAndPoors500Handler getStandardAndPoors500Handler()
    {
        if (standardAndPoorsHandler == null) {
            standardAndPoorsHandler = new StandardAndPoors500HandlerByScrape();
            standardAndPoorsHandler.setYearEndPriceHandler(getYearEndPriceHandler());
        }
        return standardAndPoorsHandler;
    }
    
    private static NasdaqHandlerByScrape nasdaqHandler;
    public static NasdaqHandler getNasdaqHandler()
    {
        if (nasdaqHandler == null) {
            nasdaqHandler = new NasdaqHandlerByScrape();
            nasdaqHandler.setYearEndPriceHandler(getYearEndPriceHandler());
        }
        return nasdaqHandler;
    }
    
    private static GoldHandlerByScrape goldHandler;
    public static GoldHandler getGoldHandler()
    {
        if (goldHandler == null) {
            goldHandler = new GoldHandlerByScrape();
            goldHandler.setCalendarHandler(getCalendarHandler());
            goldHandler.setCmeGroupHandler(getCmeGroupHandler());
            goldHandler.setYearEndPriceHandler(getYearEndPriceHandler());
        }
        return goldHandler;
    }
    
    private static FuturesExpirationsHandlerBySql futuresExpirationsHandler;
    public static FuturesExpirationsHandler getFuturesExpirationsHandler()
    {
        if (futuresExpirationsHandler == null) {
            futuresExpirationsHandler = new FuturesExpirationsHandlerBySql();
            futuresExpirationsHandler.setDataSource(getDataSource());
            futuresExpirationsHandler.setCalendarHandler(getCalendarHandler());
        }
        return futuresExpirationsHandler;
    }
    
    
    private static DowHandlerByScrape dowHandler;
    public static DowHandler getDowHandler()
    {
        if (dowHandler == null) {
            dowHandler = new DowHandlerByScrape();
            dowHandler.setYearEndPriceHandler(getYearEndPriceHandler());
        }
        return dowHandler;
    }
    
    
    private static CrudeOilHandlerByScrape crudeOilHandler;
    public static CrudeOilHandler getCrudeOilHandler()
    {
        if (crudeOilHandler == null) {
            crudeOilHandler = new CrudeOilHandlerByScrape();
            crudeOilHandler.setCalendarHandler(getCalendarHandler());
            crudeOilHandler.setFuturesExpirationsHandler(getFuturesExpirationsHandler());
            crudeOilHandler.setYearEndPriceHandler(getYearEndPriceHandler());
        }
        return crudeOilHandler;
    }
    
    private static CalendarHandlerByJava calendarHandler;
    public static CalendarHandler getCalendarHandler()
    {
        if (calendarHandler == null) {
            calendarHandler = new CalendarHandlerByJava();
        }
        return calendarHandler;
    }
    
    private static EmailAddressHandlerBySql emailAddressHandler;
    public static EmailAddressHandler getEmailAddressHandler()
    {
        if (emailAddressHandler == null) {
            emailAddressHandler = new EmailAddressHandlerBySql();
            emailAddressHandler.setDataSource(getDataSource());
        }
        return emailAddressHandler;
    }
    
    private static MessageHandlerBySmtp messageHandler;
    public static MessageHandler getMessageHandler()
    {
        if (messageHandler == null) {
            messageHandler = new MessageHandlerBySmtp();
            messageHandler.setCalendarHandler(getCalendarHandler());
            messageHandler.setEmailAddressHandler(getEmailAddressHandler());
        }
        return messageHandler;
    }
}
