/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.bean;

/**
 *
 * @author mjr2586
 */
public abstract class Market 
{
    public static enum Code {
        dow, sp, nas, cl, gc;
    }
    
    private double priceAtEndOfDay;
    private String priceChangeSinceYesterday;
    private String percentageChangeSinceYesterday;
    private double priceAtEndOfLastYear;
    private String name;


    /**
     * @return the value
     */
    public double getPriceAtEndOfDay() {
        return priceAtEndOfDay;
    }

    /**
     * @param value the value to set
     */
    public void setPriceAtEndOfDay(double value) {
        this.priceAtEndOfDay = value;
    }

    /**
     * @return the valueChange
     */
    public String getPriceChangeSinceYesterday() {
        return priceChangeSinceYesterday;
    }

    /**
     * @param valueChange the valueChange to set
     */
    public void setPriceChangeSinceYesterday(String valueChange) {
        this.priceChangeSinceYesterday = valueChange;
    }

    /**
     * @return the percentageChange
     */
    public String getPercentageChangeSinceYesterday() {
        return percentageChangeSinceYesterday;
    }

    /**
     * @param percentageChange the percentageChange to set
     */
    public void setPercentageChangeSinceYesterday(String percentageChange) {
        this.percentageChangeSinceYesterday = percentageChange;
    }

	/**
	 * @return the setPriceAtEndOfLastYear
	 */
	public double getPriceAtEndOfLastYear() {
		return priceAtEndOfLastYear;
	}

	/**
	 * @param setPriceAtEndOfLastYear the setPriceAtEndOfLastYear to set
	 */
	public void setPriceAtEndOfLastYear(double setPriceAtEndOfLastYear) {
		this.priceAtEndOfLastYear = setPriceAtEndOfLastYear;
	}
	
	/**
	 * Calculates the change of price since the end of last year.
	 * @return 
	 */
	public double getPriceChangeSinceEndOfLastYear() {
        if ( getPriceAtEndOfLastYear() == 0.00) {
            return Double.NaN;
        } else {
            return getPriceAtEndOfDay() - getPriceAtEndOfLastYear();	
        }
	}
	
	/**
	 * Calculates the percentage of the price change since the end of last year.
	 */
	public double getPercentageChangeSinceEndOfLastYear() {
        if ( getPriceAtEndOfLastYear() == 0.00) {
            return Double.NaN;
        } else {
            return (getPriceChangeSinceEndOfLastYear() / getPriceAtEndOfLastYear()) * 100;		
        }
	}
    
    public String getName() {
        return name;
    }    
    public void setName(String name) {
        this.name = name;
    }
   
    public abstract Code getCode();
}
