/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.bean;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DecimalFormat;
import org.apache.commons.lang.StringEscapeUtils;
import org.ferris.yahoo.finance.jmce.lang.StringBufferWrapper;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class Message 
{
	private StringBufferWrapper sp;
    private DecimalFormat dformat, pformat;
	
	private String getStyle(double doub) {
		return getStyle(String.format("%f",doub));
	}
	private String getStyle(String str)
	{
		if (str.charAt(0) == '+') {
			return " style=\"text-align:right;font-family:courier;color:green;\" ";
		}
		else
		if (str.charAt(0) == '-') {
			return " style=\"text-align:right;font-family:courier;color:red;\" ";
		}
		else {
			return " style=\"text-align:right;font-family:courier;color:green;\" ";
		}
	}
	
	public Message() 
	{
		sp  = new StringBufferWrapper();	
        dformat = new DecimalFormat("#,###.00");
        pformat = new DecimalFormat("#,###.00%");
	}
    
    @Override
    public String toString() {
        return sp.toString();
    }  
	
	public void writeStart() 
	{
		sp.appendLine("<h2>Market Summary</h2>");
        sp.appendLine("<a style=\"color:blue; text-decoration:underline; font-weight:bold;\" href=\"http://finance.yahoo.com/\">Go To Yahoo! Finance</a><br /><br />");
		sp.appendLine("<table border=\"1\" cellpadding=\"3\">");
	}
	
	public void writeEnd() {
		sp.appendLine("</table>");
	}
	
	public void writeHeader(int lastYear) 
	{
		sp.appendLine("<tr>");

			sp.appendLine("<th>");
				sp.appendLine("MARKET");
			sp.appendLine("</th>");

			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("" + lastYear);
				sp.appendLine("<br />");
				sp.appendLine("CLOSING");
			sp.appendLine("</th>");

			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("DAY<br/>CLOSING");
			sp.appendLine("</th>");

			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("DAY<br/>VALUE<br/>CHANGE");
			sp.appendLine("</th>");
			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("DAY<br/>PERCENT<br/>CHANGE");
			sp.appendLine("</th>");

			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("YTD<br/>VALUE<br/>CHANGE");
			sp.appendLine("</th>");
			sp.appendLine("<th style=\"vertical-align:top;\">");
				sp.appendLine("YTD<br/>PERCENT<br/>CHANGE");
			sp.appendLine("</th>");				
		sp.appendLine("</tr>");
	}

	public void writeMarket(Market market) 
	{
        
		sp.appendLine("<tr>");
            // Market
			sp.appendLine("<td style=\"font-weight:bold;\">");
            sp.appendLine(StringEscapeUtils.escapeHtml(market.getName()));            
			sp.appendLine("</td>");		

            // 2011 closing
			sp.appendLine("<td style=\"text-align:right;font-family:courier;\">");
				sp.appendLine((market.getPriceAtEndOfLastYear() == 0.00) ? "unknown" : dformat.format(market.getPriceAtEndOfLastYear()));
			sp.appendLine("</td>");

            // Day Closing
			sp.appendLine("<td style=\"text-align:right;font-family:courier;\">");
				sp.appendLine(dformat.format(market.getPriceAtEndOfDay()));
			sp.appendLine("</td>");

            // day value change
			sp.appendLine("<td "+getStyle(market.getPriceChangeSinceYesterday())+">");
				sp.appendLine(market.getPriceChangeSinceYesterday());
			sp.appendLine("</td>");

            // day percent change
			sp.appendLine("<td "+getStyle(market.getPercentageChangeSinceYesterday())+">");
				sp.appendLine(market.getPercentageChangeSinceYesterday());
			sp.appendLine("</td>");

            // ytd value change
			sp.appendLine("<td "+getStyle(market.getPriceChangeSinceEndOfLastYear())+">");
                sp.appendLine((Double.isNaN(market.getPriceChangeSinceEndOfLastYear())) ? "unknown" : dformat.format(market.getPriceChangeSinceEndOfLastYear()));
			sp.appendLine("</td>");

            // ytd percentage change
			sp.appendLine("<td "+getStyle(market.getPercentageChangeSinceEndOfLastYear())+">");
                sp.appendLine((Double.isNaN(market.getPercentageChangeSinceEndOfLastYear())) ? "unknown" : pformat.format(market.getPercentageChangeSinceEndOfLastYear()/100.00));
			sp.appendLine("</td>");	

		sp.appendLine("</tr>");
	}

    public void setThrowable(Throwable t) {
        StringWriter sw = new StringWriter();
        PrintWriter  pw = new PrintWriter(sw);
        t.printStackTrace(pw);
        pw.flush();
        sp.appendLine(sw.toString());
        pw.close();                
    }
	
	
}
