/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.util.Calendar;
import org.apache.log4j.Logger;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class CalendarHandlerByJava implements CalendarHandler
{
	private static final Logger log = Logger.getLogger(CalendarHandlerByJava.class);
	
    @Override
    public Calendar removeTime(Calendar c)
    {
        Calendar retval = Calendar.getInstance();
        retval.setTimeInMillis(c.getTimeInMillis());;
        retval.set(Calendar.HOUR, 0);
        retval.set(Calendar.MINUTE, 0);
        retval.set(Calendar.SECOND, 0);
        retval.set(Calendar.MILLISECOND, 0);
        return retval;
    }
    
    @Override
    public Calendar getNow() 
    {
        return Calendar.getInstance();
    }
    
    @Override
	public int getThisYear() 
	{
		return getNow().get(Calendar.YEAR);
	}
	
	@Override
	public int getLastYear() 
	{
		return getThisYear() - 1;
	}	
	
	@Override
	public boolean isNowTheLastDayOfTheYear() 
	{
		Calendar calendar = getNow();
		
		boolean isDec
			= calendar.get(Calendar.MONTH) == Calendar.DECEMBER;
		
		boolean is31
			= calendar.get(Calendar.DAY_OF_MONTH) == 31;
		
		return isDec && is31;
	}
}
