/*
 * Copyright 2012 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.Future;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author mjr2958
 */
public class CmeGroupHandlerByJson implements CmeGroupHandler
{
    private static final Logger log = Logger.getLogger(CmeGroupHandlerByJson.class);
    
    @Override
    public Future getGreatestOpenInterestFutureForGold() 
    {
        try 
        {
            if (log.isInfoEnabled()) {
                log.info(String.format("ENTER getGreatestOpenInterestFutureForGold()"));
            }

            Calendar calendar
                = Calendar.getInstance();
    
            String urlString =
                String.format(
                "http://www.cmegroup.com/CmeWS/md/MDServer/V1/Venue/G/Exchange/XCEC/FOI/FUT/Product/GC?currentTime=%d&contractCDs=,GCG2,GCH2,GCJ2,GCK2,GCM2,GCQ2,GCV2,GCZ2,GCG3,GCJ3,GCM3,GCQ3,GCV3,GCZ3,GCM4,GCZ4,GCM5,GCZ5,GCM6,GCZ6,GCM7,GCZ7,GCG2,GCH2,GCJ2,GCK2,GCM2,GCQ2,GCV2,GCZ2,GCG3,GCJ3,GCM3,GCQ3,GCV3,GCZ3,GCM4,GCZ4,GCM5,GCZ5,GCM6,GCZ6,GCM7,GCZ7"
                , calendar.getTimeInMillis()
            );

            if (log.isInfoEnabled()) {
                log.info(String.format("URL String: %s",urlString));
            }

            if (log.isInfoEnabled()) {
                log.info(String.format("Open connection"));
            }
            URL url = new URL(urlString);
            URLConnection conn = url.openConnection();
            InputStream is = conn.getInputStream();
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Read InputSteam into String"));
            }
            String jsonString
                = IOUtils.toString(is);
            
            if (log.isInfoEnabled()) {
                log.info(String.format("String is \"%s\"",jsonString));
            }
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Create JSONParser"));
            }
            JSONParser parser
                = new JSONParser();
            
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Parse JSON String into JSONObject"));
            }
            JSONObject jsonObject 
                = (JSONObject)parser.parse(jsonString);
            if (jsonObject == null) {throw new RuntimeException("Parser did not create a JSONObject");}
            
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Get key \"marketDataInfoAsStringList\""));
            }
            JSONObject marketDataInfoAsStringList
                = (JSONObject)jsonObject.get("marketDataInfoAsStringList");
            if (marketDataInfoAsStringList == null) {throw new RuntimeException("marketDataInfoAsStringList is null");}
            
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Get key \"market\""));
            }
            JSONArray message
                = (JSONArray)marketDataInfoAsStringList.get("message");
            if (message == null) {throw new RuntimeException("message is null");}
            
            
            int greatestOpenInterest
                = getGreatestOpenInterest(message);
            if (log.isInfoEnabled()) {
                log.info(String.format("Greatest Open Interest: %d\n", greatestOpenInterest));
            }
         
            List<JSONObject> matching
                = getMatchingGreatestOpenInterest(message, greatestOpenInterest);
            if (matching.isEmpty()) {throw new RuntimeException("No matching greatest open interest found!");}
            
            Future future = new Future(
                  StringUtils.trimToEmpty((String)matching.get(0).get("ticker"))
                , greatestOpenInterest
            );
            
            if (future.getTicker().isEmpty()) {
                throw new RuntimeException("Value for property \"ticker\" is empty");
            }
            
            return future;
        }
        catch (RuntimeException e) {
            throw e;
        }
        catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }
    }
    
    private List<JSONObject> getMatchingGreatestOpenInterest(JSONArray jsonArray, int greatestOpenInterest)
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getMatchingGreatestOpenInterest(%s,%d)",jsonArray,greatestOpenInterest));
        }
        
        List<JSONObject> matches = new LinkedList<JSONObject>();
        for (Object obj : jsonArray) 
        {
            JSONObject jsonObj = (JSONObject) obj;
            String openInterest = (String)jsonObj.get("openInterest");
            if (!openInterest.equals("-")) {
                try {
                    int openInterestInt = Integer.parseInt(openInterest);
                    if (openInterestInt == greatestOpenInterest) {
                        matches.add(jsonObj);
                    }
                } catch (Throwable ignore) {}
            }                
        }
        
        return matches;
    }
    
    private int getGreatestOpenInterest(JSONArray jsonArray)
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getGreatestOpenInterest(%s)",jsonArray));
        }
        int greatest = 0;
        for (Object obj : jsonArray) 
        {
            JSONObject jsonObj = (JSONObject) obj;
            String openInterest = (String)jsonObj.get("openInterest");
            if (!openInterest.equals("-")) {
                try {
                    int openInterestInt = Integer.parseInt(openInterest);
                    greatest = Math.max(greatest, openInterestInt);
                } catch (Throwable ignore) {}
            }                
        }
        
        return greatest;
    }
    
}
