/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.text.DecimalFormat;
import java.util.Calendar;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.FuturesExpirations;
import org.ferris.yahoo.finance.jmce.bean.Oil;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;

/**
 *
 * @author mjr2586
 */
public class CrudeOilHandlerByScrape extends AbstractHandlerByScrape implements CrudeOilHandler
{
    private static final Logger log = Logger.getLogger(CrudeOilHandlerByScrape.class);
    
    @Override
    public Oil getByCurrentContract() 
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getByCurrentContract()"));
        }
                
		try {
			FuturesExpirations futures
				= getFuturesExpirationsHandler().getForCrudeOil();
			
			if (futures == null) {
				throw new RuntimeException(
					String.format("A futures last trading date could not be found for OIL")
				);					
			}
			
			String yahooTickerSymbol
				= String
					.format("%s%s%d.nym",futures.getTickerSymbol(), futures.getContractMonth(), futures.getContractYear())
					.toLowerCase()
			;
			
			HtmlPageContents pageContents
				= getHtmlPageContents(String.format("http://finance.yahoo.com/q?s=%s",yahooTickerSymbol));

			String value = pageContents.getContentsOfTagByIds(String.format("yfs_l10_%s",yahooTickerSymbol));
			String valueChange = pageContents.getContentsOfTagByIds(String.format("yfs_c10_%s",yahooTickerSymbol));
			String percentageChange = pageContents.getContentsOfTagByIds(String.format("yfs_p20_%s",yahooTickerSymbol));

            valueChange = String.format("%s%s",getMarketDown(pageContents),valueChange);
            percentageChange = String.format("%s%s",getMarketDown(pageContents),percentageChange);
            
			if (log.isInfoEnabled()) {
				log.info(String.format("Create Oil bean and set data"));
			}
			Oil bean = new Oil();
                        bean.setName(String.format("%s (%s)",bean.getName(),yahooTickerSymbol));
			bean.setPriceAtEndOfDay(DecimalFormat.getNumberInstance().parse(value).doubleValue());
			bean.setPriceChangeSinceYesterday(valueChange);
			bean.setPercentageChangeSinceYesterday(percentageChange);
			
			YearEndPrice yearEndPrice
				= getYearEndPriceHandler().getYearEndPriceForOil(
					getCalendarHandler().getLastYear()
				);
			
			if (log.isInfoEnabled()) {
				log.info(String.format("Year end price \"%s\"", String.valueOf(yearEndPrice)));
			}			
			if (yearEndPrice != null) {
				bean.setPriceAtEndOfLastYear(yearEndPrice.getPrice().doubleValue());
			}

            return bean;
            
		} catch (Throwable t) {
			log.fatal("Couldn't get oil by current contract", t);
			throw new RuntimeException(t);
		}
    }
    
	private FuturesExpirationsHandler futuresExpirationsHandler;
    private CalendarHandler calendarHandler;

	/**
	 * @return the futuresExpirationsHandler
	 */
	public FuturesExpirationsHandler getFuturesExpirationsHandler() {
		return futuresExpirationsHandler;
	}

	/**
	 * @param futuresExpirationsHandler the futuresExpirationsHandler to set
	 */
	public void setFuturesExpirationsHandler(FuturesExpirationsHandler futuresExpirationsHandler) {
		this.futuresExpirationsHandler = futuresExpirationsHandler;
	}

    /**
     * @return the calendarHandler
     */
    public CalendarHandler getCalendarHandler() {
        return calendarHandler;
    }

    /**
     * @param calendarHandler the calendarHandler to set
     */
    public void setCalendarHandler(CalendarHandler calendarHandler) {
        this.calendarHandler = calendarHandler;
    }
}
