/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import org.ferris.yahoo.finance.jmce.bean.Dow;
import java.text.DecimalFormat;
import java.util.Calendar;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;

/**
 *
 * @author mjr2586
 */
public class DowHandlerByScrape extends AbstractHandlerByScrape implements DowHandler
{
    private static final Logger log = Logger.getLogger(DowHandlerByScrape.class);
    
    @Override
    public Dow getById(String id) 
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getById(%s)",id));
        }
                
		try {
			HtmlPageContents pageContents
				= getHtmlPageContents(id);

			String value = pageContents.getContentsOfTagByIds("yfi_rt_quote_summary_rt_top", "yfs_l10_^dji");
			String valueChange = pageContents.getContentsOfTagByIds("yfi_rt_quote_summary_rt_top","yfs_c10_^dji");
			String percentageChange = pageContents.getContentsOfTagByIds("yfi_rt_quote_summary_rt_top","yfs_p20_^dji");
            
            valueChange = String.format("%s%s",getMarketDown(pageContents),valueChange);
            percentageChange = String.format("%s%s",getMarketDown(pageContents),percentageChange);
            

			if (log.isInfoEnabled()) {
				log.info(String.format("Create Dow bean and set data"));
			}
			Dow dow = new Dow();
			dow.setPriceAtEndOfDay(DecimalFormat.getNumberInstance().parse(value).doubleValue());
			dow.setPriceChangeSinceYesterday(valueChange);
			dow.setPercentageChangeSinceYesterday(percentageChange);
			
			YearEndPrice yearEndPrice
				= getYearEndPriceHandler().getYearEndPriceForDow(
					Calendar.getInstance().get(Calendar.YEAR) - 1
				);
			
			if (log.isInfoEnabled()) {
				log.info(String.format("Year end price \"%s\"", String.valueOf(yearEndPrice)));
			}			
			if (yearEndPrice != null) {
				dow.setPriceAtEndOfLastYear(yearEndPrice.getPrice().doubleValue());
			}
				

			return dow;
		} catch (Throwable t) {
			log.fatal("Couldn't get dow by id", t);
			throw new RuntimeException(t);
		}
    }
}
