/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.EmailAddress;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class EmailAddressHandlerBySql extends AbstractHandlerBySql implements EmailAddressHandler
{
	private static final Logger log = Logger.getLogger(EmailAddressHandlerBySql.class);
    
    @Override
	public EmailAddress getAdmin()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getAdmin()"));
		}	
		
        EmailAddress.Type theType 
            = EmailAddress.Type.admin;        
		
        List<EmailAddress> list 
            = getEmailAddresses(theType);
		
        if (list.isEmpty()) {
            throw new RuntimeException(
                String.format("An \"%s\" address cannot be found in the database", theType)
            );
        }
        if (list.size() > 1) {
            throw new RuntimeException(
                String.format("Multiple type=\"%s\" addresses found in the database", theType)
            );
        }
        
        return list.get(0);
	}
    
    @Override
	public List<EmailAddress> getCc()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getCc()"));
		}	
		
        EmailAddress.Type theType 
            = EmailAddress.Type.cc;        
		
        List<EmailAddress> list 
            = getEmailAddresses(theType);
		
        return list;
	}
	
    @Override
	public List<EmailAddress> getBcc()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getBcc()"));
		}	
		
        EmailAddress.Type theType 
            = EmailAddress.Type.bcc;        
		
        List<EmailAddress> list 
            = getEmailAddresses(theType);
		        
        return list;
	}
    
    @Override
	public List<EmailAddress> getTo()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getTo()"));
		}	
		
        EmailAddress.Type theType 
            = EmailAddress.Type.to;        
		
        List<EmailAddress> list 
            = getEmailAddresses(theType);
		
        if (list.isEmpty()) {
            throw new RuntimeException(
                String.format("No type=\"%s\" addresses can be found in the database", theType)
            );
        }

        return list;
	}
    
	@Override
	public EmailAddress getFrom()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getFrom()"));
		}	
		
        EmailAddress.Type theType = null;
        
        theType = EmailAddress.Type.from;        
		List<EmailAddress> list = getEmailAddresses(theType);
		
        if (list.isEmpty()) {
            theType = EmailAddress.Type.reply;
            list = getEmailAddresses(theType);                    
        }
        
        if (list.isEmpty()) {
            throw new RuntimeException(
                String.format("A \"from\" address cannot be found in the database")
            );
        }
        if (list.size() > 1) {
            throw new RuntimeException(
                String.format("Multiple type=\"%s\" addresses found in the database", theType)
            );
        }
        
        return list.get(0);
	}
	
	
	@Override
	public EmailAddress getReplyTo()
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getReplyTo()"));
		}	
		
        EmailAddress.Type theType = null;
            
        
        theType = EmailAddress.Type.reply;        
		List<EmailAddress> list = getEmailAddresses(theType);
		
        if (list.isEmpty()) {
            theType = EmailAddress.Type.from;
            list = getEmailAddresses(theType);                    
        }
        
        if (list.isEmpty()) {
            throw new RuntimeException(
                String.format("A \"reply to\" address cannot be found in the database")
            );
        }
        if (list.size() > 1) {
            throw new RuntimeException(
                String.format("Multiple type=\"%s\" addresses found in the database", theType)
            );
        }
        
        return list.get(0);
	}
	

	private List<EmailAddress> getEmailAddresses(EmailAddress.Type type) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getEmailAddresses(%s)",type.toString()));
		}
		
		StringBuilder sp = new StringBuilder();
		sp.append(String.format(" select "));
		sp.append(String.format("		ea.id, ea.created_on, ea.updated_on, ea.type, ea.address, ea.name "));
		sp.append(String.format(" from "));
		sp.append(String.format("   email_addresses ea "));
		sp.append(String.format(" where "));
        sp.append(String.format("       ea.type=? "));
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		List<EmailAddress> beans = null;
		try {
			conn = getConnection();
			stmt = prepareStatement(conn, sp.toString());
			
			setString(stmt,1,type.toString());
			
			rs = executeQuery(stmt);				
			beans = getBeans(rs);			
		}
		finally {
			close(conn, stmt, rs);
		}	
		return beans;
	}
		
    
    private List<EmailAddress> getBeans(ResultSet rs)
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Enter getBeans(..)"));
        }
        
        List<EmailAddress> beans
            = new LinkedList<EmailAddress>();
        
        try {
            while (rs.next())
            {
                beans.add(getBean(rs));
            }
        } catch (Throwable t) {
			throw new RuntimeException("Problem getting EmailAddress bean", t);
		}
     
        return beans;
    }
	
	
	private EmailAddress getBean(ResultSet rs) 
    throws SQLException
	{
        if (log.isInfoEnabled()) {
            log.info(String.format("Enter getBean(..)"));
        }
		EmailAddress bean = null;
						
        String id 
            = rs.getString("id");

        Calendar createdOn = Calendar.getInstance();
        createdOn.setTimeInMillis(rs.getTimestamp("created_on").getTime());

        Calendar updatedOn = Calendar.getInstance();
        createdOn.setTimeInMillis(rs.getTimestamp("updated_on").getTime());

        EmailAddress.Type type
            = EmailAddress.Type.valueOf(rs.getString("type"));

        String address
            = rs.getString("address");

        String name
            = rs.getString("name");
        
        bean 				
            = new EmailAddress(
                id, createdOn, updatedOn, type, address, name
        );	
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Built bean=\"%s\" with id=\"%s\"",bean.getClass().getSimpleName(),bean.getId()));
        } 
		
		return bean;
	}
	
}
