/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import org.ferris.yahoo.finance.jmce.bean.Gold;

/**
 *
 * @author mjr2586
 */
public interface GoldHandler 
{
	/**
	 * This handler needs to determine what is the current contract by 
	 * figuring how today's date falls within the the last trading (LT)
	 * days setup for the year for Oil.  This is determined by finding
	 * the first LT day which is >= to today.  That LT day will be associated
	 * with a month and it is that month which is currently under contract.
	 * Once the contract month is know, the future's ticker symbol can be
	 * determined by:
	 * 
	 *		"Futures tickers are a little different from stocks. Each futures 
	 *		market has a ticker symbol that is followed by symbols for the 
	 *		contract month and the year. For example, crude oil futures have a
	 *		ticker symbol - CL. The complete ticker symbol for December 2007 
	 *		Crude Oil Futures would be - CLZ7.
	 *	
	 *		The "CL" stands for the underlying futures contract.
	 *		The "Z" stands for a December delivery month. 
	 *			(F=Jan, G=Feb, H=Mar, J=Apr, K=May, M=June, N=July, Q=Aug, U=Sep, V=Oct, X=Nov, Z=Dec)
	 *		The "7" stands for the year – 2007." 
	 * 
	 * An email should be automatically set near the end of the year to lookup 
	 * the dates for next year so they can be inserted into the database.  This
	 * site may be good:
	 *
	 *  #1 http://www.cmegroup.com/trading/metals/precious/gold_product_calendar_options.html
	 * 
	 * If Not, a Google search of "comex gold expiry dates 2012" should help
	 * find the information
	 * 
	 * NOTE:	Last Trading Day (LT)
	 *			First Notice Day (FN)
	 */
    Gold getByCurrentContract();
}
