/*
 * Copyright 2012 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.EmailAddress;
import org.ferris.yahoo.finance.jmce.bean.Message;
import org.ferris.yahoo.finance.jmce.mail.SimpleMailMessage;

/**
 *
 * @author mjr2958
 */
public class MessageHandlerBySmtp implements MessageHandler
{
    private static final Logger log = Logger.getLogger(MessageHandlerBySmtp.class);
    
    private EmailAddressHandler emailAddressHandler;
    private CalendarHandler calendarHandler;
    
    @Override
    public void mailAdmin(Message message) 
    {
        EmailAddress from = getEmailAddressHandler().getFrom();
        EmailAddress replyTo = getEmailAddressHandler().getReplyTo();
        EmailAddress to = getEmailAddressHandler().getAdmin();
        
        Date now
           = calendarHandler.getNow().getTime();
        
        SimpleMailMessage mailer
            = new SimpleMailMessage("localhost");
        
        mailer.setSendDate(now);
        mailer.setFrom(from);
        mailer.setReplyTo(replyTo);
        mailer.addTo(to);
        mailer.setSubject(
            String.format(
                "Yahoo! Finance JMCE Scrapper Error on %s",
                new SimpleDateFormat("MM/dd").format(now)
            )
        );
        mailer.setText(message.toString());
        mailer.send();
    }
            
    @Override
    public void mailMessage(Message message) 
    {
        EmailAddress from = getEmailAddressHandler().getFrom();
        EmailAddress replyTo = getEmailAddressHandler().getReplyTo();
        List<EmailAddress> to = getEmailAddressHandler().getTo();
        List<EmailAddress> bcc = getEmailAddressHandler().getBcc();
        List<EmailAddress> cc = getEmailAddressHandler().getCc();

        Date now
           = calendarHandler.getNow().getTime();
        
        SimpleMailMessage mailer
            = new SimpleMailMessage("localhost");
        
        mailer.setSendDate(now);
        mailer.setFrom(from);
        mailer.setReplyTo(replyTo);
        mailer.addTo(to.toArray(new EmailAddress[0]));
        mailer.addBcc(bcc.toArray(new EmailAddress[0]));
        mailer.addCc(cc.toArray(new EmailAddress[0]));
        mailer.setSubject(
            String.format(
                "Daily Market Summary for %s",
                new SimpleDateFormat("MM/dd").format(now)
            )
        );
        mailer.setText(message.toString());
        mailer.send();
    }

    /**
     * @return the emailAddressHandler
     */
    public EmailAddressHandler getEmailAddressHandler() {
        return emailAddressHandler;
    }

    /**
     * @param emailAddressHandler the emailAddressHandler to set
     */
    public void setEmailAddressHandler(EmailAddressHandler emailAddressHandler) {
        this.emailAddressHandler = emailAddressHandler;
    }

    /**
     * @return the calendarHandler
     */
    public CalendarHandler getCalendarHandler() {
        return calendarHandler;
    }

    /**
     * @param calendarHandler the calendarHandler to set
     */
    public void setCalendarHandler(CalendarHandler calendarHandler) {
        this.calendarHandler = calendarHandler;
    }
    
}
