/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.text.DecimalFormat;
import java.util.Calendar;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.Nasdaq;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;

/**
 *
 * @author mjr2586
 */
public class NasdaqHandlerByScrape extends AbstractHandlerByScrape implements NasdaqHandler
{
    private static final Logger log = Logger.getLogger(NasdaqHandlerByScrape.class);
    
    @Override
    public Nasdaq getById(String id) 
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getById(%s)",id));
        }
                
		try {
			HtmlPageContents pageContents
				= getHtmlPageContents(id);

			String value = pageContents.getContentsOfTagByIds("yfs_l10_^ixic");
			String valueChange = pageContents.getContentsOfTagByIds("yfs_c10_^ixic","yfs_c10_^ixic");
			String percentageChange = pageContents.getContentsOfTagByIds("yfs_p20_^ixic","yfs_p20_^ixic");
            
            valueChange = String.format("%s%s",getMarketDown(pageContents),valueChange);
            percentageChange = String.format("%s%s",getMarketDown(pageContents),percentageChange);
            
			if (log.isInfoEnabled()) {
				log.info(String.format("Create Nasdaq bean and set data"));
			}
			Nasdaq bean = new Nasdaq();
			bean.setPriceAtEndOfDay(DecimalFormat.getNumberInstance().parse(value).doubleValue());
			bean.setPriceChangeSinceYesterday(valueChange);
			bean.setPercentageChangeSinceYesterday(percentageChange);
			
			
			YearEndPrice yearEndPrice
				= getYearEndPriceHandler().getYearEndPriceForNasdaq(
					Calendar.getInstance().get(Calendar.YEAR) - 1
				);
			
			if (log.isInfoEnabled()) {
				log.info(String.format("Year end price \"%s\"", String.valueOf(yearEndPrice)));
			}
			if (yearEndPrice != null) {
				bean.setPriceAtEndOfLastYear(yearEndPrice.getPrice().doubleValue());
			}
				

			return bean;
		} catch (Throwable t) {
			log.fatal("Couldn't get nasdaq by id", t);
			throw new RuntimeException(t);
		}
    }    
}
