package org.ferris.yahoo.finance.jmce.mail;

import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.Message.RecipientType;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.ferris.yahoo.finance.jmce.bean.EmailAddress;

@SuppressWarnings("restriction")
public class SimpleMailMessage
{
    private Session             sendMailSession;
    private MimeMessage         message;
    private String              body;
    private Date                date;
    
    private void addAddresses(RecipientType type, EmailAddress...to)
    {
        try {
            if (to != null) {
                for (EmailAddress address : to) {
                    message.addRecipient(
                        type, new InternetAddress(address.getAddress(),address.getName())
                    );   
                }
            }
        } catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }        
    }    
    
    private void sendAsText() { this.send(false); }
    
    private void sendAsHtml() { this.send(true); }
    
    private void send(boolean asHtml)
    {
        try {
            message.setSentDate(date);
            if (asHtml) {
                message.setHeader("X-Mailer", "sendhtml");
                message.setDataHandler(
                    new DataHandler(new ByteArrayDataSource(body, "text/html"))
                );
            } else {
                message.setText(body);
            }
            //Transport transport;
            //transport = m_sendMailSession.getTransport("smtp");
            Transport.send(message);
        } catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }
    }    

    /**
     * Class constructor.
     *
     * @exception Exception
     *  Throws an exception if the mail.properties
     * file that specifies the SMTP server cannot be found or read.
     */
    public SimpleMailMessage(String smtpServer)    
    {
        Properties props = new Properties();
        props.put("mail.smtp.host", smtpServer);
        sendMailSession = Session.getInstance(props, null);
        message = new MimeMessage(sendMailSession);
    }

    /**
     * 
     */
    public void setFrom(EmailAddress from)
    {
        try {
            message.setFrom(new InternetAddress(from.getAddress(), from.getName()));
        } catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }
    }
    
    /**
     * 
     */
    public void setReplyTo(EmailAddress from)
    {
        try {
            message.setReplyTo(new Address[]{new InternetAddress(from.getAddress(), from.getName())});
        } catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }
    }

    /**
     * 
     */
    public void addTo(EmailAddress...to)
    {
        addAddresses(RecipientType.TO, to);
    }
    
    /**
     * 
     */
    public void addCc(EmailAddress...cc)
    {
        addAddresses(RecipientType.CC, cc);
    }
    
    /**
     * 
     */
    public void addBcc(EmailAddress...bcc)
    {
        addAddresses(RecipientType.BCC, bcc);
    }     

    /**
     * 
     */
    public void setSubject(String in_subject)
    {
        try {
            message.setSubject(in_subject);
        } catch (Throwable t) {
            throw new RuntimeException(t.getMessage(),t);
        }
    }

    /**
     * 
     */
    public void setText(String in_text) {
        body = in_text;
    }    
    
    /**
     * 
     */
    public void setSendDate(Date date) {
        this.date = date;
    }
    
    /**
     * 
     */
    public void send() {
        sendAsHtml();
    }
    
}