package org.ferris.yahoo.finance.jmce.main;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.version.Version;
import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.impl.StdSchedulerFactory;

/**
 * Starts market and securities.
 * 
 * @author mjremij
 *
 */
public class Main
{
	private final Logger log = Logger.getLogger(getClass());

	public static final void main (String [] args) { 
		new Main(args);
	}
	
	public Main(String [] args)	
	{
		try {
			init(args);
		} catch (Throwable t) {
			log.fatal(t.getMessage(), t);
		}
	}
		
	public void init(String [] args)
	throws Exception
	{
		log.info("Starting:  Yahoo! Finance JMCE            ");
		log.info("Version: " + new Version().getVersion());
	        
//        log.info("--nojmce Execute immediately.");
//		new MainJob().execute(null);        
//        if (true) { return;}
                
		if (args.length == 0)
		{
			log.info("No command line arguments...scheduling.");
			
			// Create a CronTrigger with the given name, group and expression.
			log.info("Create Trigger");
			CronTrigger trigger 
				= new CronTrigger(
					  "Yahoo! Finance JMCE Trigger" // name
					, null // group
					, "0 30 15 ? * MON-FRI" // schedule
				);
			
			log.info("Create Scheduler");
	        Scheduler scheduler 
	        	= StdSchedulerFactory.getDefaultScheduler();
	
			log.info("Create JobDetail");
			JobDetail jobDetail 
				= new JobDetail(
					  MainJob.class.getSimpleName() // name
					, null // group
					, MainJob.class // class
				);
			
			log.info("Schedule job");
			scheduler.scheduleJob(jobDetail, trigger);
			
	        log.info("Start scheduler");
	        scheduler.start();
		}
		else
		{
			log.info("args: " + Arrays.asList(args));
			
			if ("--nojmce".equals(args[0])) 
			{
				log.info("--nojmce Execute immediately.");
				new MainJob().execute(null);
			}
			else {
				log.info("No command.");
				log.info("[DONE]");
			}
		}
		
	}
	
}
