package org.ferris.yahoo.finance.jmce.main;


import java.io.InputStream;
import java.util.Properties;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.BeanFactory;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.Message;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

public class MainJob implements Job
{
	private final Logger log = Logger.getLogger(getClass());
		
	public MainJob() {
        if (log.isInfoEnabled()) {
            log.info(String.format("MainJob created"));
        }
	}
	

	public void execute(JobExecutionContext arg0) 
	throws JobExecutionException 
	{
        try {
            properties();        

            // scrape
            Market dow = dow();
            Market nas = nasdaq();
            Market sp  = sp();
            Market gold= gold();
            Market oil = oil();

            // save daily data
            saveDailyData(
                dow,nas,sp,gold,oil
            );

            // save yearly data
            saveYearlyData(
                dow,nas,sp,gold,oil
            );

            // email        
            email(
                message(dow,nas,sp,gold,oil)
            );
        } catch (Throwable t) {
            error(t);
        }
	}
    
    private void error(Throwable t)
    {
        try {
            Message message
                = new Message();
            message.setThrowable(t);
            BeanFactory.getMessageHandler().mailAdmin(message);            
        } catch (Throwable ignore) {
            log.fatal(ignore, t);
        }
    }
    
    private void saveDailyData(Market...markets)
    {
        for (Market m : markets) {
            BeanFactory.getDayEndPriceHandler().setDayEndPriceForMarket(
                  m.getPriceAtEndOfDay()
                , BeanFactory.getCalendarHandler().getNow()
                , m.getCode()
            );
        }
    }
    
    private void saveYearlyData(Market...markets)
    {
        for (Market m : markets) {
            BeanFactory.getYearEndPriceHandler().setYearEndPriceForMarket(
                  m.getPriceAtEndOfDay()
                , BeanFactory.getCalendarHandler().getThisYear()
                , m.getCode()
            );
        }
    }
    
    private void email(Message message)
    {
        BeanFactory.getMessageHandler().mailMessage(message);
    }
    
    private Message message(Market...markets)
    {
        Message message
			= new Message();
	
		message.writeStart();	
		message.writeHeader(BeanFactory.getCalendarHandler().getLastYear());        
        
        for (Market m : markets) {
            message.writeMarket(m);
        }
        
        message.writeEnd();
		
		log.info(message.toString());
        
        return message;
    }
    
    
    private Market dow()
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Scrape for DOW"));
        }
        Market market
            = BeanFactory.getDowHandler().getById("http://finance.yahoo.com/q?s=^dji");
        return market;
    }
    
    
    
    private Market nasdaq()
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Scrape for NASDAQ"));
        }
        Market market
            = BeanFactory.getNasdaqHandler().getById("http://finance.yahoo.com/q?s=^ixic");
        return market;
    }    
    
    
    private Market sp()
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Scrape for S&P"));
        }
        Market market
            = BeanFactory.getStandardAndPoors500Handler().getById("http://finance.yahoo.com/q?s=^gspc");
        return market;
    }   
    
        
    private Market gold()
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Scrape for Gold"));
        }
        Market market
            = BeanFactory.getGoldHandler().getByCurrentContract();
        return market;
    }     
    
    
    private Market oil()
    {
        if (log.isInfoEnabled()) {
            log.info(String.format("Scrape for Oil"));
        }
        Market market
            = BeanFactory.getCrudeOilHandler().getByCurrentContract();
        return market;
    }     
    
    private void properties()
    {
        String resource = "/ferris-yahoo-finance-jmce-private.properties";
        InputStream is = null;
        try {
            is = getClass().getResourceAsStream(resource);
            Properties props = new Properties();
            props.load(is);
            for (String key : props.stringPropertyNames()) {
                String val = props.getProperty(key);
                System.getProperties().setProperty(key, val);
            }
        } catch (Throwable t) {
            throw new RuntimeException(
                String.format("Problem getting resource \"%s\" as stream",resource)        
                , t
            );
        } finally {
            IOUtils.closeQuietly(is);
        }
        
    }
}
