/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package integration.handler;

import java.util.UUID;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.ferris.yahoo.finance.jmce.handler.PrimaryKeyHandler;
import java.sql.DriverManager;
import java.util.Properties;
import java.sql.Connection;
import javax.sql.DataSource;
import org.ferris.yahoo.finance.jmce.handler.AbstractHandlerBySql;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class AbstractHandlerBySqlIT {
	
	private Connection connection;
	
	protected DataSource dataSource;
	
	private AbstractHandlerBySql abstractHandlerBySql;
    
    private PrimaryKeyHandler primaryKeyHandler;
    
    protected PrimaryKeyHandler getPrimaryKeyHandler() {
        return primaryKeyHandler;
    }
	
	public AbstractHandlerBySqlIT() {
	}

	@BeforeClass
	public static void setUpClass() throws Exception {
	}

	@AfterClass
	public static void tearDownClass() throws Exception {
	}
	
	@Before
	public void setUp() throws Exception
	{
		// JDBC Connection
		{
			String url = "jdbc:postgresql://localhost/MARKET_INTEGRATION";
			Properties props = new Properties();
			props.setProperty("user","integration");
			props.setProperty("password","integration");
			//props.setProperty("ssl","true");
			connection = spy(DriverManager.getConnection(url, props));
			doNothing()
				.when(connection)
				.close();
		}
		
		// DataSource
		{
			dataSource
				= mock(DataSource.class);

			doReturn(connection)
				.when(dataSource)
				.getConnection();
		}
				
		// Abstract Handler
		{
			abstractHandlerBySql
				= spy(new AbstractHandlerBySql())
			;

			doReturn(dataSource)
				.when(abstractHandlerBySql)
				.getDataSource()
			;
		}
        
        // Primary Key Handler
        {
        
            class PrimaryKeyAnswer implements Answer<String>
            {

                @Override
                public String answer(InvocationOnMock invocation) throws Throwable {
                    
                    String uuid = UUID.randomUUID().toString();
                    String prefix = "integration";                            
                    return String.format(
                        "%s%s",prefix, uuid.substring(uuid.indexOf("-")).substring(0,36-prefix.length())
                    );
                }
                
            }
            
            primaryKeyHandler
                = mock(PrimaryKeyHandler.class);
            doAnswer(new PrimaryKeyAnswer())
                .when(primaryKeyHandler)
                .getPrimaryKey()
            ;
        }
	}
	
	@After
	public void tearDown() throws Exception
	{
		connection.close();
	}
	
	@Test
	public void getConnection() 
	throws Exception
	{
		assertNotNull("Why is JUnit Connection null?", abstractHandlerBySql.getConnection());
	}
	
	
	@Test
	public void getDataSource() 
	throws Exception
	{
		assertNotNull("Why is JUnit DataSource null?", abstractHandlerBySql.getDataSource());		
	}
}
