/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package integration.handler;

import org.ferris.yahoo.finance.jmce.bean.DayEndPrice;
import java.util.Calendar;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.junit.Test;
import org.ferris.yahoo.finance.jmce.handler.DayEndPriceHandlerBySql;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandler;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class DayEndPriceHandlerBySqlIT extends AbstractHandlerBySqlIT 
{	
	private DayEndPriceHandlerBySql dayEndPriceHandler;
	private CalendarHandler calendarHandler;
    
    @Test
    public void saveDayEndPriceAndMakeSureCreatedOnHasATimeValue()
    {
        String market = "gold";
        double close = 888;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.OCTOBER);
        day.set(Calendar.DAY_OF_MONTH, 30);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.gc);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForGold(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        Calendar createdOn = price.getCreatedOn();
        int hour = createdOn.get(Calendar.HOUR);
        int mins = createdOn.get(Calendar.MINUTE);
        int secs = createdOn.get(Calendar.SECOND);
        int mils = createdOn.get(Calendar.MILLISECOND);        
        int sum = hour+mins+secs+mils;        
        assertTrue("Sum of hours, mins, secs, milliseconds, should be greater than 0", (sum > 0));        
    }
    
    @Test
    public void saveDayEndPriceForGoldWhenThereIsAlreadyAPriceForGoldToday()
    {        
        String market = "gold";
        double close = 3333.33;
        
        // 3/8/2001 - THIS DATE IS IN before.sql WITH PRICE OF 999.99
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 8);
        day.set(Calendar.YEAR, 2012);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.gc);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForGold(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    
    /*
	 void (double price, Calendar day);
	 void (double price, Calendar day);
     */        
    @Test
    public void saveDayEndPriceForGold()
    {
        String market = "gold";
        double close = 1647;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 13);
        day.set(Calendar.YEAR, 2001);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.gc);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForGold(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void saveDayEndPriceForOil()
    {
        String market = "oil";
        double close = 94.63;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 13);
        day.set(Calendar.YEAR, 2001);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.cl);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForOil(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void saveDayEndPriceForStandardAndPoors500()
    {
        String market = "s&p";
        double close = 569.01;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 13);
        day.set(Calendar.YEAR, 2001);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.sp);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForStandardAndPoors500(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void saveDayEndPriceForNasdaq()
    {
        String market = "nasdaq";
        double close = 2563.00;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 13);
        day.set(Calendar.YEAR, 2001);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.nas);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForNasdaq(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void saveDayEndPriceForDow()
    {
        String market = "dow";
        double close = 8001.00;
        
        // 3/15/2001
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.MARCH);
        day.set(Calendar.DAY_OF_MONTH, 13);
        day.set(Calendar.YEAR, 2001);
        
        dayEndPriceHandler
            .setDayEndPriceForMarket(close, day, Market.Code.dow);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForDow(day);
        assertNotNull(String.format("Day end price not inserted for %s",market),price);  
        assertEquals(close, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void getDayEndPriceForStandardAndPoors500()
    {
        String market = "s&p";
        
        // 1/1/2012
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.FEBRUARY);
        day.set(Calendar.DAY_OF_MONTH, 1);
        day.set(Calendar.YEAR, 2012);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForStandardAndPoors500(day);
        assertNotNull(String.format("No day end price found for %s",market),price);  
        assertEquals(562.32, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void getDayEndPriceForOil()
    {
        String market = "oil";
        
        // 1/1/2012
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.FEBRUARY);
        day.set(Calendar.DAY_OF_MONTH, 1);
        day.set(Calendar.YEAR, 2012);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForOil(day);
        assertNotNull(String.format("No day end price found for %s",market),price);  
        assertEquals(85.6, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void getDayEndPriceForNasdaq()
    {
        String market = "nasdaq";
        
        // 1/1/2012
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.FEBRUARY);
        day.set(Calendar.DAY_OF_MONTH, 1);
        day.set(Calendar.YEAR, 2012);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForNasdaq(day);
        assertNotNull(String.format("No day end price found for %s",market),price);  
        assertEquals(2014, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void getDayEndPriceForGold()
    {
        String market = "gold";
        
        // 1/1/2012
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.FEBRUARY);
        day.set(Calendar.DAY_OF_MONTH, 1);
        day.set(Calendar.YEAR, 2012);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForGold(day);
        assertNotNull(String.format("No day end price found for %s",market),price);  
        assertEquals(1525, price.getPrice().doubleValue(), 0.00);
    }
    
    @Test
    public void getDayEndPriceForDow()
    {
        String market = "dow";
        
        // 1/1/2012
        Calendar day = Calendar.getInstance();
        day.set(Calendar.MONTH, Calendar.FEBRUARY);
        day.set(Calendar.DAY_OF_MONTH, 1);
        day.set(Calendar.YEAR, 2012);
        
        DayEndPrice price
            = dayEndPriceHandler.getDayEndPriceForDow(day);
        assertNotNull(String.format("No day end price found for %s",market),price); 
        assertEquals(12217.56, price.getPrice().doubleValue(), 0.00);
    }
	
	public DayEndPriceHandlerBySqlIT() {
	}

	@BeforeClass
	public static void setUpClass() throws Exception {
	}

	@AfterClass
	public static void tearDownClass() throws Exception {
	}
	
	@Before
	public void setUp() throws Exception 
	{		
		// Important!
		super.setUp(); 
		
		// Handler
		{            
			calendarHandler
				= spy(new CalendarHandlerByJava());
						
			dayEndPriceHandler
				= spy(new DayEndPriceHandlerBySql())
			;
			
			doReturn(dataSource)
				.when(dayEndPriceHandler)
				.getDataSource()
			;
			
			doReturn(calendarHandler)
				.when(dayEndPriceHandler)
				.getCalendarHandler()
			;
            
            doReturn(getPrimaryKeyHandler())
                .when(dayEndPriceHandler)
                .getPrimaryKeyHandler()
            ;
		}
	}
	
	@After
	public void tearDown() throws Exception 
	{
		// Important!
		super.tearDown(); 
	}
}
