/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package integration.handler;

import java.util.Comparator;
import java.util.Collections;
import java.util.List;
import org.ferris.yahoo.finance.jmce.bean.EmailAddress;
import org.ferris.yahoo.finance.jmce.handler.EmailAddressHandlerBySql;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class EmailAddressHandlerBySqlIT extends AbstractHandlerBySqlIT 
{
    @Test
	public void getAdmin() 
	throws Exception
	{
		EmailAddress address
            = emailAddressHandler.getAdmin();
        		
		assertNotNull("EmailAddress should not be null",address);
        
        assertEquals(EmailAddress.Type.admin, address.getType());
        assertEquals("vivian.violet@rainbow.net", address.getAddress());
        assertEquals("Vivian V. Violet", address.getName());        
	}
    
    @Test
	public void getReplyTo() 
	throws Exception
	{
		EmailAddress address
            = emailAddressHandler.getReplyTo();
        		
		assertNotNull("EmailAddress should not be null",address);
        
        assertEquals(EmailAddress.Type.reply, address.getType());
        assertEquals("yahoo-finance-jmce@ferris.sourceforge.net", address.getAddress());
        assertEquals("Yahoo! Finance JMCE", address.getName());        
	}
    
    @Test
	public void getFrom() 
	throws Exception
	{
		EmailAddress address
            = emailAddressHandler.getFrom();
        		
		assertNotNull("EmailAddress should not be null",address);
        
        assertEquals(EmailAddress.Type.from, address.getType());
        assertEquals("yahoo-finance-jmce@ferris.sourceforge.net", address.getAddress());
        assertEquals("Yahoo! Finance JMCE", address.getName());        
	}
    
    @Test
	public void getTo() 
	throws Exception
	{
		List<EmailAddress> list
            = emailAddressHandler.getTo();
        		
		assertNotNull("List of EmailAddress should not be null",list);
		assertEquals(2, list.size());	
        
        Collections.sort(list, new Comparator<EmailAddress>(){
            @Override
            public int compare(EmailAddress o1, EmailAddress o2) {
                return o1.getAddress().compareTo(o2.getAddress());
            }
        });
        
        EmailAddress address = list.get(0);        
        assertEquals(EmailAddress.Type.to, address.getType());
        assertEquals("oscar.o@prodigy.net", address.getAddress());
        assertEquals("Oscar Orange", address.getName());               
        
        address = list.get(1);        
        assertEquals(EmailAddress.Type.to, address.getType());
        assertEquals("rita.red@prodigy.net", address.getAddress());
        assertEquals("Rita Red", address.getName());              
	}
     
    @Test
	public void getBcc() 
	throws Exception
	{
		List<EmailAddress> list
            = emailAddressHandler.getBcc();
        		
		assertNotNull("List of EmailAddress should not be null",list);
		assertEquals(1, list.size());	
        
        EmailAddress address
            = list.get(0);
        
        assertEquals(EmailAddress.Type.bcc, address.getType());
        assertEquals("greg.green@prodigy.net", address.getAddress());
        assertEquals("Greg Green", address.getName());        
	}
    
    
	@Test
	public void getCc() 
	throws Exception
	{
		List<EmailAddress> list
            = emailAddressHandler.getCc();
        		
		assertNotNull("List of EmailAddress should not be null",list);
		assertEquals(1, list.size());	
        
        EmailAddress address
            = list.get(0);
        
        assertEquals(EmailAddress.Type.cc, address.getType());
        assertEquals("yet.yello@prodigy.net", address.getAddress());
        assertEquals("Yet", address.getName());
	}
	
	
	
    ////////////////////////////////////////////////////////////////////////////
    //
    // test
    //
    ////////////////////////////////////////////////////////////////////////////
	private EmailAddressHandlerBySql emailAddressHandler;
	
	public EmailAddressHandlerBySqlIT() {
	}

	@BeforeClass
	public static void setUpClass() throws Exception {
	}

	@AfterClass
	public static void tearDownClass() throws Exception {
	}
	
	@Before
	public void setUp() throws Exception 
	{		
		// Important!
		super.setUp(); 
		
		// Handler
		{
			emailAddressHandler
				= spy(new EmailAddressHandlerBySql())
			;
			
			doReturn(dataSource)
				.when(emailAddressHandler)
				.getDataSource()
			;						
		}
	}
	
	@After
	public void tearDown() throws Exception 
	{
		// Important!
		super.tearDown(); 
	}
}
