/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package integration.handler;

import org.ferris.yahoo.finance.jmce.handler.CalendarHandler;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import java.util.Calendar;
import java.util.GregorianCalendar;
import org.ferris.yahoo.finance.jmce.bean.FuturesExpirations;
import org.ferris.yahoo.finance.jmce.handler.FuturesExpirationsHandlerBySql;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class FuturesExpirationsHandlerBySqlIT extends AbstractHandlerBySqlIT 
{
	@Test
	public void getForCrudeOilInJanuary2012() 
	throws Exception
	{
		FuturesExpirations bean
			= futuresExpirationHandler.getForCrudeOil();
		
		assertNotNull(bean);
		
		assertTrue(String.format("Should start with \"integration-\" but is \"%s\"", bean.getId()), bean.getId().startsWith("integration-"));
		assertEquals("crude oil for jan 2012", bean.getContractDescription());
		assertEquals("f", String.format("%s",bean.getContractMonth()));
		assertEquals(12, bean.getContractYear());
		GregorianCalendar calendar = new GregorianCalendar(2011, Calendar.DECEMBER, 19);
		assertEquals(calendar.getTime(), bean.getLastTradingDate());		
	}
	
	
	
	private FuturesExpirationsHandlerBySql futuresExpirationHandler;
    private CalendarHandler calendarHandler;
	
	public FuturesExpirationsHandlerBySqlIT() {
	}

	@BeforeClass
	public static void setUpClass() throws Exception {
	}

	@AfterClass
	public static void tearDownClass() throws Exception {
	}
	
	@Before
	public void setUp() throws Exception 
	{		
		// Important!
		super.setUp(); 
		
		// Handler
		{
            calendarHandler
				= spy(new CalendarHandlerByJava());
            
            doReturn(new GregorianCalendar(2011, Calendar.DECEMBER, 13))
				.when(calendarHandler)
				.getNow()
			;
            
			futuresExpirationHandler
				= spy(new FuturesExpirationsHandlerBySql())
			;

			doReturn(dataSource)
				.when(futuresExpirationHandler)
				.getDataSource()
			;
            doReturn(calendarHandler)
				.when(futuresExpirationHandler)
				.getCalendarHandler()
			;
            
//			doReturn(new GregorianCalendar(2011, Calendar.DECEMBER, 13).getTime())
//				.when(futuresExpirationHandler)
//				.getNow()
//			;
		}
	}
	
	@After
	public void tearDown() throws Exception 
	{
		// Important!
		super.tearDown(); 
	}
}
