/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package integration.handler;

import java.util.Calendar;
import org.ferris.yahoo.finance.jmce.bean.DayEndPrice;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandler;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandlerBySql;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class YearEndPriceHandlerBySqlIT extends AbstractHandlerBySqlIT 
{
    @Test
    public void saveYearEndPriceAndMakeSureCreatedOnAndUpdatedOnHasATimeValue()
    {
        double price = 1230.65;
		int year  = 99;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.gc);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForGold(year);
		
        {
            Calendar calendar = yep.getCreatedOn();
            int hour = calendar.get(Calendar.HOUR);
            int mins = calendar.get(Calendar.MINUTE);
            int secs = calendar.get(Calendar.SECOND);
            int mils = calendar.get(Calendar.MILLISECOND);        
            int sum = hour+mins+secs+mils;        
            assertTrue("For createdOn, sum of hours, mins, secs, milliseconds, should be greater than 0", (sum > 0));        
        }
        {
            Calendar calendar = yep.getUpdatedOn();
            int hour = calendar.get(Calendar.HOUR);
            int mins = calendar.get(Calendar.MINUTE);
            int secs = calendar.get(Calendar.SECOND);
            int mils = calendar.get(Calendar.MILLISECOND);        
            int sum = hour+mins+secs+mils;        
            assertTrue("For updatedOn, sum of hours, mins, secs, milliseconds, should be greater than 0", (sum > 0));        
        }
    }
    
    
	@Test
	public void setYearEndPriceForGold() 
	throws Exception
	{
		double price = 1230.65;
		int year  = 10;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.gc);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForGold(year);
		
		assertNotNull("Year end price not found for Gold",yep);
		assertEquals(price, yep.getPrice().doubleValue(), 0.00);		
	}
	
	@Test
	public void setYearEndPriceForOil() 
	throws Exception
	{
		double price = 89.56;
		int year  = 11;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.cl);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForOil(year);
		
		assertNotNull("Year end price not found for Oil",yep);
		assertEquals(price, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	@Test
	public void setYearEndPriceForStandardAndPoors500() 
	throws Exception
	{
		double price = 458.78;
		int year  = 12;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.sp);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForStandardAndPoors500(year);
		
		assertNotNull("Year end price not found for S&P500",yep);
		assertEquals(price, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	@Test
	public void setYearEndPriceForNasdaq() 
	throws Exception
	{
		double price = 1569.35;
		int year  = 13;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.nas);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForNasdaq(year);
		
		assertNotNull("Year end price not found for Nasdaq",yep);
		assertEquals(price, yep.getPrice().doubleValue(), 0.00);		
	}
	
	@Test
	public void setYearEndPriceForDow() 
	throws Exception
	{
		double price = 77.67;
		int year  = 14;
		yearEndPriceHandler.setYearEndPriceForMarket(price, year, Market.Code.dow);
		
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForDow(year);
		
		assertNotNull("Year end price not found for Dow",yep);
		assertEquals(price, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	@Test
	public void getYearEndPriceForOil() 
	throws Exception
	{
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForOil(8888);
		
		assertNotNull(yep);
		assertEquals(98.56, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	@Test
	public void getYearEndPriceForGold() 
	throws Exception
	{
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForGold(5757);
		
		assertNotNull(yep);
		assertEquals(1236.0, yep.getPrice().doubleValue(), 0.00);		
	}
	
	@Test
	public void getYearEndPriceForStandardAndPoors500() 
	throws Exception
	{
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForStandardAndPoors500(7895);
		
		assertNotNull(yep);
		assertEquals(987.76, yep.getPrice().doubleValue(), 0.00);		
	}
	
	@Test
	public void getYearEndPriceForNasdaq() 
	throws Exception
	{
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForNasdaq(3030);
		
		assertNotNull(yep);
		assertEquals(123.45, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	@Test
	public void getYearEndPriceForDow() 
	throws Exception
	{
		YearEndPrice yep
			= yearEndPriceHandler.getYearEndPriceForDow(3030);
		
		assertNotNull(yep);
		assertEquals(100.00, yep.getPrice().doubleValue(), 0.00);		
	}
	
	
	private YearEndPriceHandlerBySql yearEndPriceHandler;
	private CalendarHandler calendarHandler;
	
	public YearEndPriceHandlerBySqlIT() {
	}

	@BeforeClass
	public static void setUpClass() throws Exception {
	}

	@AfterClass
	public static void tearDownClass() throws Exception {
	}
	
	@Before
	public void setUp() throws Exception 
	{		
		// Important!
		super.setUp(); 
		
		// Handler
		{
            
			calendarHandler
				= spy(new CalendarHandlerByJava());
			doReturn(true)
				.when(calendarHandler)
				.isNowTheLastDayOfTheYear()
			;
			
			yearEndPriceHandler
				= spy(new YearEndPriceHandlerBySql())
			;
			
			doReturn(dataSource)
				.when(yearEndPriceHandler)
				.getDataSource()
			;
			
			doReturn(calendarHandler)
				.when(yearEndPriceHandler)
				.getCalendarHandler()
			;
            
            doReturn(getPrimaryKeyHandler())
                .when(yearEndPriceHandler)
                .getPrimaryKeyHandler()
            ;
		}
	}
	
	@After
	public void tearDown() throws Exception 
	{
		// Important!
		super.tearDown(); 
	}
}
