/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.io.IOUtils;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;
import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class AbstractHandlerByScrapeTest 
{
	public AbstractHandlerByScrapeTest() 
	{
    }
	
	public InputStream getInputStream(String fullyQualifiedResourceName)
    {
        final InputStream is = getClass().getResourceAsStream(fullyQualifiedResourceName);
        if (is == null) {
            fail(String.format("HTML resource \"%s\" not found",fullyQualifiedResourceName));
        }
        return is;
    }
    
	public Calendar getCalendar()
	{
		Calendar calendar = Calendar.getInstance();
		
		// Date 05 Dec 2011
		calendar.set(Calendar.MONTH, Calendar.DECEMBER);
		calendar.set(Calendar.DATE, 5);
		calendar.set(Calendar.YEAR, 2011);
		
		// Time 10:04:15.250 PM the HOUR_OF_DAY is 22. 
		calendar.set(Calendar.HOUR_OF_DAY, 22);
		calendar.set(Calendar.MINUTE, 4);
		calendar.set(Calendar.SECOND, 15);
		calendar.set(Calendar.MILLISECOND, 250);
		
		return calendar;		
	}
	
	public HtmlPageContents getHtmlPageContents(String classPathResource)
	{
		// This class path resource contains the HTML page I want to get	
		//String classPathResource = "/2011-11-14/dji.html";
		
		// Create new object
		HtmlPageContents hpc = new HtmlPageContents();
		
		// Attempt to convert the class path resource into a String and
		// save the string into the HtmlPageContents object just created.
		try {
			hpc.setPageContentsAsHtml(
				IOUtils.toString(getInputStream(classPathResource))
			 );						
		} catch (IOException e) {
			throw new RuntimeException(
				String.format("Problem getting class path resource \"%s\"",classPathResource)
			);
		}	
		
		return hpc;
	}
	
	
	public YearEndPriceHandler getYearEndPriceHandler(double price)
	{
		String id = "aaklsjf-348n4-23h9";
		Calendar createdOn = getCalendar();
		Calendar updatedOn = getCalendar();
		String marketId = "marketId";
		int year = 3089;
		BigDecimal bd = new BigDecimal(price);
		
		YearEndPrice yearEndPrice 
			= new YearEndPrice(id, createdOn, updatedOn, marketId, year, bd);
			
        YearEndPriceHandler mock
			= mock(YearEndPriceHandler.class);	
		
		doReturn(yearEndPrice)
			.when(mock)
			.getYearEndPriceForDow(anyInt())
		;	
		doReturn(yearEndPrice)
			.when(mock)
			.getYearEndPriceForNasdaq(anyInt())
		;	
		doReturn(yearEndPrice)
			.when(mock)
			.getYearEndPriceForStandardAndPoors500(anyInt())
		;
		doReturn(null)
			.when(mock)
			.getYearEndPriceForOil(anyInt())
		;
		
		return mock;
	}

    
	@Test
	public void testSpy()
	{
		List list = new LinkedList();
		List spy = spy(list);
		
		//when(spy.size()).thenReturn(100);
		doReturn(10).when(spy).size();		
		assertEquals(10, spy.size());
		
		spy.add("a");
		assertEquals(10, spy.size());		
		assertEquals("a", spy.get(0));
	}	

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }         
}
