/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;

import java.util.GregorianCalendar;
import integration.handler.AbstractHandlerBySqlIT;
import java.util.Calendar;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author mjr2586
 */
public class CalendarHandlerByJavaTest extends AbstractHandlerBySqlIT 
{	
    @Test
    public void isNowTheLastDayOfTheYear_Yes()
    {
        Calendar c = new GregorianCalendar(2012, Calendar.DECEMBER, 31);
        doReturn(c)
            .when(calendarHandler)
            .getNow()
        ;
        
        assertTrue("31 Dec 2012 should BE the last day of the year", calendarHandler.isNowTheLastDayOfTheYear());        
    }
    
    @Test
    public void isNowTheLastDayOfTheYear_No()
    {
        Calendar c = new GregorianCalendar(2012, Calendar.JANUARY, 13);
        doReturn(c)
            .when(calendarHandler)
            .getNow()
        ;
        
        assertFalse("13 Jan 2012 should NOT be the last day of the year", calendarHandler.isNowTheLastDayOfTheYear());        
    }
    
    @Test
	public void getLastYear() 
	throws Exception
	{
		int year = Calendar.getInstance().get(Calendar.YEAR) - 1;	
		assertEquals(year, calendarHandler.getLastYear());
	}
	
	
	@Test
	public void getThisYear() 
	throws Exception
	{
		int year = Calendar.getInstance().get(Calendar.YEAR);	
		assertEquals(year, calendarHandler.getThisYear());
	}
	
	
	private CalendarHandlerByJava calendarHandler;	
    
	public CalendarHandlerByJavaTest() {}

	@BeforeClass
	public static void setUpClass() throws Exception {}

	@AfterClass
	public static void tearDownClass() throws Exception {}
	
	@Before
	public void setUp() throws Exception 
	{		
		// Important!
		super.setUp(); 
		
		// Handler
		{
			calendarHandler
				= spy(new CalendarHandlerByJava());
			;
		}
	}
	
	@After
	public void tearDown() throws Exception 
	{
		// Important!
		super.tearDown(); 
	}
}
