/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;
import org.ferris.yahoo.finance.jmce.bean.FuturesExpirations;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.Oil;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandler;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import org.ferris.yahoo.finance.jmce.handler.CrudeOilHandlerByScrape;
import org.ferris.yahoo.finance.jmce.handler.FuturesExpirationsHandler;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class CrudeOilHandlerByScrapeTest extends AbstractHandlerByScrapeTest
{
    
    public CrudeOilHandlerByScrapeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
     
    
    @Test
    public void getById_For_2012_03_27_NeitherUpOrDown()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-03-27/oil-up-or-down.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid db queries
		FuturesExpirationsHandler feh
			= mock(FuturesExpirationsHandler.class);
		doReturn(new FuturesExpirations(null, null, null, "cl", "crude oil for 2012 may", 'k', 12, null))
			.when(feh)
			.getForCrudeOil()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		CrudeOilHandlerByScrape spy
			= spy(new CrudeOilHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getFuturesExpirationsHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(feh)
			.when(spy)
			.getFuturesExpirationsHandler()
		;
		
		// Assert I'm getting the right bean back.
		Oil bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("0.00%", bean.getPercentageChangeSinceYesterday());
		assertEquals(107.03, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("0.00", bean.getPriceChangeSinceYesterday());
        assertEquals("Oil (clk12.nym)", bean.getName());
        assertEquals(bean.getCode(), Market.Code.cl);
		
	}
    
    	
    @Test
    public void getById_For_2012_02_08_UP()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/oil-up.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid db queries
		FuturesExpirationsHandler feh
			= mock(FuturesExpirationsHandler.class);
		doReturn(new FuturesExpirations(null, null, null, "cl", "crude oil for 2012 mar", 'h', 12, null))
			.when(feh)
			.getForCrudeOil()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		CrudeOilHandlerByScrape spy
			= spy(new CrudeOilHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getFuturesExpirationsHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(feh)
			.when(spy)
			.getFuturesExpirationsHandler()
		;
		
		// Assert I'm getting the right bean back.
		Oil bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("0.25%", bean.getPercentageChangeSinceYesterday());
		assertEquals(101.16, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("0.25", bean.getPriceChangeSinceYesterday());
        assertEquals("Oil (clh12.nym)", bean.getName());
        assertEquals(bean.getCode(), Market.Code.cl);
		
	}
    
    
	@Test
    public void getById_For_2012_02_08_DOWN()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/oil-down.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid db queries
		FuturesExpirationsHandler feh
			= mock(FuturesExpirationsHandler.class);
		doReturn(new FuturesExpirations(null, null, null, "cl", "crude oil for 2012 mar", 'h', 12, null))
			.when(feh)
			.getForCrudeOil()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		CrudeOilHandlerByScrape spy
			= spy(new CrudeOilHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getFuturesExpirationsHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(feh)
			.when(spy)
			.getFuturesExpirationsHandler()
		;
		
		// Assert I'm getting the right bean back.
		Oil bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("-0.04%", bean.getPercentageChangeSinceYesterday());
		assertEquals(100.87, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-0.04", bean.getPriceChangeSinceYesterday());
        assertEquals("Oil (clh12.nym)", bean.getName());        
		
	}
    
	public void getById_For_2011_11_14()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2011-11-14/oil.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid db queries
		FuturesExpirationsHandler feh
			= mock(FuturesExpirationsHandler.class);
		doReturn(new FuturesExpirations(null, null, null, "cl", "crude oil for 2011 dec", 'z', 11, null))
			.when(feh)
			.getForCrudeOil()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		CrudeOilHandlerByScrape spy
			= spy(new CrudeOilHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getFuturesExpirationsHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(feh)
			.when(spy)
			.getFuturesExpirationsHandler()
		;
		
		// Assert I'm getting the right bean back.
		Oil bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("2.81%", bean.getPercentageChangeSinceYesterday());
		assertEquals(102.16, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("2.79", bean.getPriceChangeSinceYesterday());	
		
	}
}
