/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.ferris.yahoo.finance.jmce.bean.Dow;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.handler.DowHandlerByScrape;
import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class DowHandlerByScrapeTest extends AbstractHandlerByScrapeTest
{
    
    public DowHandlerByScrapeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
        
    @Test
	public void getById_For_2012_02_08_UP()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/dow-up.html");
                
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(9.45);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		DowHandlerByScrape scrapper
			= new DowHandlerByScrape();
		DowHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Dow dow 
			= spy.getById(String.format("anyString"));

		assertNotNull(dow);
		assertEquals("0.26%", dow.getPercentageChangeSinceYesterday());
		assertEquals(12878.20, dow.getPriceAtEndOfDay(),0.0);		
		assertEquals(9.45, dow.getPriceAtEndOfLastYear(), 0.0);
                assertEquals("33.07", dow.getPriceChangeSinceYesterday());		
	}
        
    @Test
	public void getById_For_2012_02_08_DOWN()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/dow-down.html");
                
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(9.45);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		DowHandlerByScrape scrapper
			= new DowHandlerByScrape();
		DowHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Dow dow 
			= spy.getById(String.format("anyString"));

		assertNotNull(dow);		
		assertEquals(12865.22, dow.getPriceAtEndOfDay(),0.0);		
		assertEquals(9.45, dow.getPriceAtEndOfLastYear(), 0.0);
                assertEquals("-18.73", dow.getPriceChangeSinceYesterday());
                assertEquals("-0.15%", dow.getPercentageChangeSinceYesterday());
	}
     
    			
	public void getById_For_2011_11_14()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2011-11-14/dji.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(9.45);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		DowHandlerByScrape scrapper
			= new DowHandlerByScrape();
		DowHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Dow dow 
			= spy.getById(String.format("anyString"));

		assertNotNull(dow);
		assertEquals("-0.50%", dow.getPercentageChangeSinceYesterday());
		assertEquals(12035.31, dow.getPriceAtEndOfDay(),0.0);		
		assertEquals(9.45, dow.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-60.85", dow.getPriceChangeSinceYesterday());		
        assertEquals(dow.getCode(), Market.Code.dow);
	}
        
        
}
