package junit.handler;

import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.Future;
import org.ferris.yahoo.finance.jmce.bean.Gold;
import org.ferris.yahoo.finance.jmce.handler.CalendarHandlerByJava;
import org.ferris.yahoo.finance.jmce.handler.CmeGroupHandler;
import org.ferris.yahoo.finance.jmce.handler.GoldHandlerByScrape;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class GoldHandlerByScrapeTest extends AbstractHandlerByScrapeTest
{
    
    public GoldHandlerByScrapeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    
    @Test
    public void getById_For_2012_02_08_UP()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/gold-up.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid http call
		CmeGroupHandler cgh
			= mock(CmeGroupHandler.class);
		doReturn(new Future("GCJ2",123545))
			.when(cgh)
			.getGreatestOpenInterestFutureForGold()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		GoldHandlerByScrape spy
			= spy(new GoldHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getCmeGroupHandler() method to avoid 
		// having to make an http call in test
		doReturn(cgh)
			.when(spy)
			.getCmeGroupHandler()
		;
		
		// Assert I'm getting the right bean back.
		Gold bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("0.99%", bean.getPercentageChangeSinceYesterday());
		assertEquals(1734.70, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("17.00", bean.getPriceChangeSinceYesterday());	
        assertEquals("Gold (gcj12.cmx)", bean.getName());        
		
	} 
     
    			
	@Test
    public void getById_For_2012_02_08_DOWN()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/gold-down.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(0.00);
		
		// Get mocked handler to avoid http call
		CmeGroupHandler cgh
			= mock(CmeGroupHandler.class);
		doReturn(new Future("GCJ2",123545))
			.when(cgh)
			.getGreatestOpenInterestFutureForGold()
		;
		        
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		GoldHandlerByScrape spy
			= spy(new GoldHandlerByScrape());
        
        // Return spy'd CalendarHandler
        doReturn(spy(new CalendarHandlerByJava()))
            .when(spy)
            .getCalendarHandler()
        ;
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Stub the getFuturesExpirationsHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(cgh)
			.when(spy)
			.getCmeGroupHandler()
		;
		
		// Assert I'm getting the right bean back.
		Gold bean 
			= spy.getByCurrentContract();

		assertNotNull(bean);
		assertEquals("-0.45%", bean.getPercentageChangeSinceYesterday());
		assertEquals(1717.20, bean.getPriceAtEndOfDay(),0.0);		
		assertEquals(0.00, bean.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-7.70", bean.getPriceChangeSinceYesterday());	
		assertEquals("Gold (gcj12.cmx)", bean.getName());  
        assertEquals(bean.getCode(), Market.Code.gc);
        
    }   
}
