/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.Nasdaq;
import org.ferris.yahoo.finance.jmce.handler.NasdaqHandlerByScrape;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class NasdaqHandlerByScrapeTest extends AbstractHandlerByScrapeTest
{
    
    public NasdaqHandlerByScrapeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    @Test
    public void getById_For_2012_02_08_UP()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/nas-up.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(2652.87);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		NasdaqHandlerByScrape scrapper
			= new NasdaqHandlerByScrape();
		NasdaqHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Nasdaq nasdaq 
			= spy.getById(String.format("anyString"));

		assertNotNull(nasdaq);
		assertEquals("0.02%", nasdaq.getPercentageChangeSinceYesterday());
		assertEquals(2931.83, nasdaq.getPriceAtEndOfDay(),0.0);		
		assertEquals(2652.87, nasdaq.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("0.44", nasdaq.getPriceChangeSinceYesterday());		
	}
     
    @Test
    public void getById_For_2012_02_08_DOWN()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/nas-down.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(2652.87);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		NasdaqHandlerByScrape scrapper
			= new NasdaqHandlerByScrape();
		NasdaqHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Nasdaq nasdaq 
			= spy.getById(String.format("anyString"));

		assertNotNull(nasdaq);
		assertEquals("-0.35%", nasdaq.getPercentageChangeSinceYesterday());
		assertEquals(2921.09, nasdaq.getPriceAtEndOfDay(),0.0);		
		assertEquals(2652.87, nasdaq.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-10.30", nasdaq.getPriceChangeSinceYesterday());		
	}			
	
	public void getById_For_2011_11_14()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2011-11-14/nasdaq.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(2652.87);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		NasdaqHandlerByScrape scrapper
			= new NasdaqHandlerByScrape();
		NasdaqHandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		Nasdaq nasdaq 
			= spy.getById(String.format("anyString"));

		assertNotNull(nasdaq);
		assertEquals("-0.29%", nasdaq.getPercentageChangeSinceYesterday());
		assertEquals(2678.37, nasdaq.getPriceAtEndOfDay(),0.0);		
		assertEquals(2652.87, nasdaq.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-7.83", nasdaq.getPriceChangeSinceYesterday());		
        assertEquals(nasdaq.getCode(), Market.Code.nas);
	}
}
