/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package junit.handler;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.StandardAndPoors500;
import org.ferris.yahoo.finance.jmce.handler.StandardAndPoors500HandlerByScrape;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.ferris.yahoo.finance.jmce.handler.YearEndPriceHandler;
import org.ferris.yahoo.finance.jmce.net.HtmlPageContents;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author mjr2586
 */
public class StandardAndPoors500HandlerByScrapeTest extends AbstractHandlerByScrapeTest
{
    
    public StandardAndPoors500HandlerByScrapeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
     
    @Test
    public void getById_For_2012_02_08_UP()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/sp-up.html");		
        
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(1257.64);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		StandardAndPoors500HandlerByScrape scrapper
			= new StandardAndPoors500HandlerByScrape();
		StandardAndPoors500HandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		StandardAndPoors500 sp500 
			= spy.getById(String.format("anyString"));

		assertNotNull(sp500);
		assertEquals("0.09%", sp500.getPercentageChangeSinceYesterday());
		assertEquals(1351.70, sp500.getPriceAtEndOfDay(),0.0);		
		assertEquals(1257.64, sp500.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("1.20", sp500.getPriceChangeSinceYesterday());		
	}
    			
	@Test
    public void getById_For_2012_02_08_DOWN()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2012-02-08/sp-down.html");		
        
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(1257.64);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		StandardAndPoors500HandlerByScrape scrapper
			= new StandardAndPoors500HandlerByScrape();
		StandardAndPoors500HandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		StandardAndPoors500 sp500 
			= spy.getById(String.format("anyString"));

		assertNotNull(sp500);
		assertEquals("-0.32%", sp500.getPercentageChangeSinceYesterday());
		assertEquals(1347.50, sp500.getPriceAtEndOfDay(),0.0);		
		assertEquals(1257.64, sp500.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-4.27", sp500.getPriceChangeSinceYesterday());		
	}
    
    
	public void getById_For_2011_11_14()
	{
		// Get HTML page from class path resource
		HtmlPageContents hpc
			= getHtmlPageContents("/2011-11-14/sp500.html");
		
		// Get mocked handler to avoid db queries
		YearEndPriceHandler yeph
			= getYearEndPriceHandler(1257.64);
		
		// Create a new instance of the scrapper,
		// then create an Mockito spy of it.
		StandardAndPoors500HandlerByScrape scrapper
			= new StandardAndPoors500HandlerByScrape();
		StandardAndPoors500HandlerByScrape spy
			= spy(scrapper);
		
		// Stub the getHtmlPageContents() method so
		// that whatever string is passed to it, the
		// object I created above is returned
		doReturn(hpc)
			.when(spy)
			.getHtmlPageContents(anyString())
		;
		
		// Stub the getYearEndPriceHandler() method to avoid 
		// having to call database queries in a unit test
		doReturn(yeph)
			.when(spy)
			.getYearEndPriceHandler()
		;
		
		// Assert I'm getting the right bean back.
		StandardAndPoors500 sp500 
			= spy.getById(String.format("anyString"));

		assertNotNull(sp500);
		assertEquals("-0.41%", sp500.getPercentageChangeSinceYesterday());
		assertEquals(1252.62, sp500.getPriceAtEndOfDay(),0.0);		
		assertEquals(1257.64, sp500.getPriceAtEndOfLastYear(), 0.0);
        assertEquals("-5.19", sp500.getPriceChangeSinceYesterday());		
        assertEquals(sp500.getCode(), Market.Code.sp);
	}
}
